---------------------------------------------------------------------------------- 
-- create date:    16:23:46 06/11/2012 
-- module name:    fir_transpose - behavioral 
----------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.config.all;

entity fir_transpose is
    port (
        clk      : in  std_logic;
        reset    : in  std_logic;
        syn_tick : in  std_logic; 
        din      : in  std_logic_vector (AMPL_BITS  downto 0);
        dout     : out std_logic_vector (OUT_BITS-1 downto 0) -- adds decimal places and reduces quantization error
    );
end fir_transpose;

architecture behavioral of fir_transpose is

type coef_type    is array(NCOEF-1 downto 0) of signed(COEF_BITS-1 downto 0);
signal coef     : coef_type;

type mul_type    is array(NCOEF-1 downto 0) of signed(MUL_BITS-1 downto 0);
signal mul_aux : mul_type;  

type cascade_type is array(NCOEF-1 downto 0) of signed(TOTAL_BITS-1 downto 0); 
signal cascade : cascade_type;

component filter_mul is
    port (
        clk      : in  std_logic;
        reset    : in  std_logic;
        syn_tick : in  std_logic; 
        din      : in  signed(AMPL_BITS   downto 0);
        coef     : in  signed(COEF_BITS-1 downto 0);
		dout1    : out signed(MUL_BITS-1  downto 0); 
		dout2    : out signed(MUL_BITS-1  downto 0)
   );
end component;

signal round_val : signed(TOTAL_BITS-1 downto 0) := (others => '0');
signal dinr1     : signed(din'range);  -- registrador de ios na entrada

signal dinr2     : signed(din'range) := (others => '0');  -- registrador de slices (pipeline do barramento dos registradores r2)
attribute keep   : string;
attribute keep of dinr2 : signal is "yes";
signal dout_int  : signed(dout'range) := (others => '0');  -- registrador de slices (pipeline do sinal de saida)

begin
    coef <= (
        0 => to_signed(79996,COEF_BITS),
        1 => to_signed(85648,COEF_BITS),
        2 => to_signed(91143,COEF_BITS),
        3 => to_signed(96436,COEF_BITS),
        4 => to_signed(101486,COEF_BITS),
        5 => to_signed(106251,COEF_BITS),
        6 => to_signed(110692,COEF_BITS),
        7 => to_signed(114772,COEF_BITS),
        8 => to_signed(118457,COEF_BITS),
        9 => to_signed(121717,COEF_BITS),
        10 => to_signed(124524,COEF_BITS),
        11 => to_signed(126854,COEF_BITS),
        12 => to_signed(128687,COEF_BITS),
        13 => to_signed(130007,COEF_BITS),
        14 => to_signed(130804,COEF_BITS),
        15 => to_signed(131070,COEF_BITS),
        16 => to_signed(130804,COEF_BITS),
        17 => to_signed(130007,COEF_BITS),
        18 => to_signed(128687,COEF_BITS),
        19 => to_signed(126854,COEF_BITS),
        20 => to_signed(124524,COEF_BITS),
        21 => to_signed(121717,COEF_BITS),
        22 => to_signed(118457,COEF_BITS),
        23 => to_signed(114772,COEF_BITS),
        24 => to_signed(110692,COEF_BITS),
        25 => to_signed(106251,COEF_BITS),
        26 => to_signed(101486,COEF_BITS),
        27 => to_signed(96436,COEF_BITS),
        28 => to_signed(91143,COEF_BITS),
        29 => to_signed(85648,COEF_BITS),
        30 => to_signed(79996,COEF_BITS)
--			 0 => to_signed(1, COEF_BITS),
--			 1 => to_signed(1, COEF_BITS),
--			 2 => to_signed(1, COEF_BITS),
--			 3 => to_signed(1, COEF_BITS),
--			 4 => to_signed(1, COEF_BITS)
    );

    -- registradores de entrada e saida
    process(clk)  begin
        if rising_edge(clk)  then
            if reset = '1' then
                dinr1   <= (others => '0');
                dout    <= (others => '0');
            else
                if (syn_tick = '1') then
                    dinr1 <= signed(din);
                    dout  <= std_logic_vector(dout_int);
                end if; 
            end if;
        end if;
    end process;

    -- registradores de pipeline do barramento de entrada e saida
    process(clk)  begin
        if rising_edge(clk)  then
            dinr2 <= dinr1;
            dout_int <= cascade(NCOEF-1)(cascade'length downto cascade'length-(dout'length-1));
--				dout_int <= cascade(NCOEF-1)(AMPL_BITS+7 downto 0);
		  end if;
    end process;

    gen : for i in 0 to (NCOEF-1)/2 generate
        bloco_0 : if i = 0  generate
            bl0 : filter_mul  port map (
                clk      => clk,
                reset    => reset,
                syn_tick => syn_tick,
                din      => dinr2,
                coef     => coef(0),
                dout1    => mul_aux(0), 
                dout2    => mul_aux(NCOEF-1)  
					 );		 
        end generate;

			bloco_meio : if i = (NCOEF-1)/2  generate
            bl0 : filter_mul  port map (
                clk      => clk,
                reset    => reset,
                syn_tick => syn_tick,
                din      => dinr2,
                coef     => coef(i),
                dout1    => mul_aux(i),
                dout2    => open  );
        end generate;		  
	
        bloco_x : if (i /= 0 and i /= (NCOEF-1)/2)  generate
            blx : filter_mul  port map (
                clk      => clk,
                reset    => reset,
                syn_tick => syn_tick,
                din      => dinr2,
                coef     => coef(i),
                dout1    => mul_aux(i),
                dout2    => mul_aux(NCOEF-1-i)  );
        end generate;	
    end generate;
	
	process(clk)
	begin
		if rising_edge(clk) then
			if reset = '1' then
				cascade <= (others => (others => '0') );
			elsif syn_tick = '1' then
				cascade(0) <= mul_aux(0) + round_val;
				
				for i in 1 to NCOEF-1 loop
					cascade(i) <= cascade(i-1) + mul_aux(i);
				end loop;
			end if;
			
		end if;
	end process;
end behavioral;