----------------------------------------------------------------------------------
-- Create Date:    18:01:28 09/26/2012 
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.config.all;

entity mean_dac is
    Port ( clk   : in  STD_LOGIC;
           reset : in  STD_LOGIC;
           btn   : in  STD_LOGIC;
           sw    : in  STD_LOGIC;
           mean  : out STD_LOGIC_VECTOR (RAMP_BITS-1 downto 0);
           count_mean : out integer range 0 to 100-- mean level from 0 to 9.6
           );
end mean_dac;

architecture Behavioral of mean_dac is

type state_type is (selection, ready); 
signal state_reg, state_next: state_type; 
signal s_reg, s_next: integer range 0 to 16;


begin          

-- FSMD state & data register
process(clk, reset)
begin
    if rising_edge(clk) then
        if reset = '1' then
            state_reg <= ready; 
            s_reg     <= 0; 
        else
            state_reg <= state_next;
            s_reg     <= s_next; 
        end if;
    end if;
end process; 

-- Next state logic & data path function units / routing
process(state_reg, s_reg, btn, sw)
begin
    state_next <= state_reg; 
    s_next <= s_reg; 

    case state_reg is 
        when ready =>
            if sw= '1' then
--                s_next <= 0; 
                state_next <= selection; 
            end if; 
            
        when selection =>
            if sw= '1' then 
                if (btn = '1') then
                    if (s_reg = 16) then
                        s_next <= 0; 
                    else
                        s_next <= s_reg + 1;
                    end if; 
                end if; 
            else
                state_next <= ready; 
            end if; 
    end case; 
    
end process; 

-- Output logic

with s_reg select
mean <= (others => '0') when 0,  --Nvel mdio: 0.0V - 0
        "000011111010"  when 1,  --Nvel mdio: 0.6V - 250
        "000111110100"  when 2,  --Nvel mdio: 1.2V - 500
        "001011101110"  when 3,  --Nvel mdio: 1.8V - 750
        "001111101000"  when 4,  --Nvel mdio: 2.4V - 1000
        "010011100010"  when 5,  --Nvel mdio: 3.0V - 1250
        "010111011100"  when 6,  --Nvel mdio: 3.6V - 1500
        "011011010110"  when 7,  --Nvel mdio: 4.2V - 1750        
        "011111010000"  when 8,  --Nvel mdio: 4.8V - 2000       
        "100011001010"  when 9,  --Nvel mdio: 5.4V - 2250        
        "100111000100"  when 10, --Nvel mdio: 6.0V - 2500       
        "101010111110"  when 11, --Nvel mdio: 6.6V - 2750
        "101110111000"  when 12, --Nvel mdio: 7.2V - 3000       
        "110010110010"  when 13, --Nvel mdio: 7.8V - 3250
        "110110101100"  when 14, --Nvel mdio: 8.4V - 3500       
        "111010100110"  when 15, --Nvel mdio: 9.0V - 3750        
        "111110100000"  when 16, --Nvel mdio: 9.6V - 4000
        "000000000000"  when others;
            
with s_reg select
count_mean    <= 0   when 0,  --Nvel mdio: 0.0V
            6   when 1,  --Nvel mdio: 0.6V
            12  when 2,  --Nvel mdio: 1.2V
            18  when 3,  --Nvel mdio: 1.8V
            24  when 4,  --Nvel mdio: 2.4V
            30  when 5,  --Nvel mdio: 3.0V
            36  when 6,  --Nvel mdio: 3.6V
            42  when 7,  --Nvel mdio: 4.2V
            48  when 8,  --Nvel mdio: 4.8V
            54  when 9,  --Nvel mdio: 5.4V
            60  when 10, --Nvel mdio: 6.0V
            66  when 11, --Nvel mdio: 6.6V
            72  when 12, --Nvel mdio: 7.2V
            78  when 13, --Nvel mdio: 7.8V
            84  when 14, --Nvel mdio: 8.4V
            90  when 15, --Nvel mdio: 9.0V
            96  when 16, --Nvel mdio: 9.6V
            0  when others;
            
           
end Behavioral;
